// module sfml2.cpp (SFML with opengl) - version for Linux/Debian
// original sample program from the Internet:
//   https://www.sfml-dev.org/documentation/2.4.2/

// strongly enhanced by EH 08-2017
// compile on windows with g++ (Mingw):
//   see makefile sfml2.mak
// for execution: PATH must be set to include sfml-graphics-2.dll:
//  on Windows: set PATH=%PATH%;c:\huckert\src\SFML\SFML-2.4.2\bin
//
// compile on Linux/Debian with GNU g++ :
//   see makefile sfml2u.mak
//   libs must have been installed via: sudo apt-get install libsfml-dev

#include <SFML/Audio.hpp>
#include <SFML/Graphics.hpp>

#include <iostream>
#include <inttypes.h>  // not really needed

// ---------------------------------------------------
// define five horizontal lines corresponding to a music system
sf::VertexArray makeNotenzeile(int xp1, int xp2, int yp)
{
  //
  #define LDIST 10
  #define LINES 5
  // Horiz.line 1
  sf::VertexArray system(sf::Lines, LINES * 2);
  //
  int m = 0;
  for (int n=0; n < LINES; n++)
  {
    system[m].position   = sf::Vector2f(xp1, yp + (n * LDIST));
    system[m].color      = sf::Color::Yellow;
    system[m+1].position = sf::Vector2f(xp2, yp + (n * LDIST));
    system[m+1].color    = sf::Color::Yellow;
    m += 2;
  }
  //
  return system;
}   // end makeNotenzeile()

// --------------------------------------------------
int main()
{
  int ret;
  sf::Clock clock;
  sf::Time elapsed;
  int64_t t64;
  unsigned long count = 0L;
  #define WINW 1000
  #define WINH 800  
  //
  // Create the main window
  sf::RenderWindow window(sf::VideoMode(WINW, WINH), "SFML window");
  //
  // Load a sprite to display
  sf::Texture texture;
  //if (!texture.loadFromFile("cute_image.jpg"))
  sf::IntRect area(100, 100, 600, 450);  // defines the area to be loaded
  //ret = texture.loadFromFile("./Koala.jpg", area);
  ret = texture.loadFromFile("./Koala.jpg");
  if (! ret)
  {
    
    std::cout << "file Koala.jpg not found" << std::endl;
    return EXIT_FAILURE;
  }
  texture.setSmooth(true);
  sf::Sprite sprite(texture);
  //
  // load a second sprite (violin key)
  sf::Image violKey;
  ret = violKey.loadFromFile("./keyv2.bmp");
  if (! ret)
  {
    std::cout << "file keyv2 not found" << std::endl;
    return EXIT_FAILURE;
  }
  sf::Color col(255,255,255);
  violKey.createMaskFromColor(col, 20);
  sf::Texture violKeyTx;
  violKeyTx.loadFromImage(violKey);
  sf::Sprite violKeySp(violKeyTx);
  //
  // Create a graphical text to display
  sf::Font font;
  // EH: path added for font
  #ifdef WIN32
  ret = font.loadFromFile("c:\\windows\\fonts\\arial.ttf");
  #else
  ret = font.loadFromFile("/usr/share/fonts/truetype/dejavu/DejaVuSans-Bold.ttf");
  #endif
  if (! ret)
  {
    std::cout << "font DejaVuSans-Bold.ttf not found" << std::endl;
    return EXIT_FAILURE;
  }
  sf::Text text("Hello SFML", font, 45);
  text.setPosition(500, 70);
  text.setColor(sf::Color::Green);
  text.setRotation(360.0 - 15.0);
  //
  // Load a music file to play
  /*
  sf::Music music;
  if (!music.openFromFile("./nice_music.ogg"))
      return EXIT_FAILURE;
  //
  // Play the music
  music.play();
  */
  //
  // rectangles are the replacement for a thick line
  sf::RectangleShape rectangle;
  rectangle.setSize(sf::Vector2f(100, 80));
  rectangle.setOutlineColor(sf::Color::Red);
  rectangle.setOutlineThickness(5);
  rectangle.setPosition(30, 70);
  //
  // Define five music systems
  sf::VertexArray system1 = makeNotenzeile(10, WINW - 10, (WINH / 2) + 0);
  sf::VertexArray system2 = makeNotenzeile(10, WINW - 10, (WINH / 2) + 80);
  sf::VertexArray system3 = makeNotenzeile(10, WINW - 10, (WINH / 2) + 160);
  sf::VertexArray system4 = makeNotenzeile(10, WINW - 10, (WINH / 2) + 240);
  sf::VertexArray system5 = makeNotenzeile(10, WINW - 10, (WINH / 2) + 320);
  //
  float rot = 0.0;
  //
  // Start the loop
  while (window.isOpen())
  {
      // Process events
      sf::Event event;
      while (window.pollEvent(event))
      {
          // Close window: exit
          if (event.type == sf::Event::Closed)
          {
            std::cout << "Exit required" << std::endl;
            window.close();
          }
      }
      //
      clock.restart();
      // Clear screen
      window.clear();
      //
      // Draw the sprite
      window.draw(sprite);
      //
      // Draw the string
      window.draw(text);
      //
      window.draw(rectangle);
      //
      // draw the music systems (5 horizontal lines per system)
      window.draw(system1);
      violKeySp.setPosition(10, (WINH / 2) - 8);
      window.draw(violKeySp);
      window.draw(system2);
      window.draw(system3);
      window.draw(system4);
      window.draw(system5);
      //
      // Update the window
      window.display();
      //
      // result on Windows: mostly < 1 ms, occasionally 5 ms
      elapsed = clock.restart();
      t64 = elapsed.asMicroseconds();
      //std::cout << "count=" << count << " t=" << t64 << std::endl;
      count++;
      if ((count % 100) == 0)
      {
        // rotate the sprite/texture
        rot = rot + 1.0;
        if (rot > 90.0)
          rot = 0.0;
        //sprite.setRotation(rot);
      }
  }
  return EXIT_SUCCESS;
}   // end main()
